import { ViteSSG } from 'vite-ssg'
import { createPinia } from 'pinia'
const hljs = import('highlight.js')
import type { HLJSApi } from 'highlight.js'
const marked = import('marked')
import { markedHighlight } from 'marked-highlight'

import type { RoutedWindow, SiteGlobals } from '@goldenwere/mackenzii-types'

import main from './main.vue'
import './main.sass'

import { createRoutes, initializeRouteStore } from './routes'
import { headingSectionsExtension } from './utilities/marked'

import globals from 'content/config.json'

declare const window: RoutedWindow

export const createApp = ViteSSG(
  // the root component
  main,
  // vue-router options
  { routes: createRoutes(globals as unknown as SiteGlobals) },
  // function to have custom setups
  async ({ app, router, routes, isClient, initialState }) => {
    const hljsResolved: HLJSApi = await hljs as any
    const markedResolved = await marked
    markedResolved
    .use(headingSectionsExtension() as any)
    .use(markedHighlight({
      langPrefix: 'hljs language-',
      highlight(code, lang) {
        const language = hljsResolved.getLanguage(lang) ? lang : 'plaintext'
        return hljsResolved.highlight(code, { language }).value
      },
    }))

    if (isClient) {
      import('@goldenwere/mackenzii-embeds').then(({ registerLinkEmbed }) => {
        registerLinkEmbed()
      })
      window.router = router
    }

    app.use(createPinia())
    initializeRouteStore(routes, globals as unknown as SiteGlobals)
  },
)
